﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

using System.IO;
using System.Net;

using DsInstaller.model;
using DsInstaller.behaviour;

using DsShared;

// Includes this project:
// http://dotnetzip.codeplex.com/

namespace ds_installer
{
    class Program
    {
        static void Main(string[] args)
        {
			DsShared.NetUtil.initialize();

			NetworkCredential credential = DsShared.NetUtil.Credential;
            DsShared.Status status = new DsShared.Status();

            // Get the list of packages -- ideally use whatever was supplied in args,
            // otherwise gather them from the standard package directory.
            string[] input = args;
            // debugging
//			input = new string[1] { @"C:\work\downstream\ds_release_manager\docs\examples\installer_example.xml" };
			// Default to the predefined directory (obviously setup for my work)
            if (input == null || input.Length < 1)
            {
                string myDocs = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                string installPath = Path.Combine(myDocs, Path.Combine("downstream", "install"));
                input = new string[1] { installPath };
            }
			// Get all the packages to install, based on an array that can contain files and folders.
            Package[] packages = Package.ReadXmlAllFrom(input);

            if (packages == null || packages.Length < 1)
            {
                Console.WriteLine("no input");
                return;
            }
            
            Console.WriteLine("DsInstaller on remote host " + DsShared.NetUtil.HOST);

            // Locate all my remote files
			Locator locator = new Locator(credential);
			forEachApp(packages, a => locator.on(a));

			// Download each remote file
			Downloader downloader = new Downloader(credential, status);
			forEachApp(packages, a => downloader.on(a));

			// Install each downloaded file
			Installer installer = new Installer(status);
			forEachApp(packages, a => installer.on(a));

			Console.WriteLine("installer finished");
			Thread.Sleep(5000);
		}

		// Perform an action on each app in each package
		static private void forEachApp(Package[] packages, Action<App> f)
		{
			foreach (Package p in packages)
			{
				if (p != null)
				{
					foreach (App a in p.mApp)
					{
						f.Invoke(a);
					}
				}
			}
		}
    }
}
