﻿using System;
using System.Collections.Generic;
using System.IO;

using DsInstaller.model;
using DsShared;

using Ionic.Zip;

namespace DsInstaller.behaviour
{
	// Perform the install operation on a downloaded file
	class Installer
	{
        DsShared.Status mStatus = null;

        public Installer(DsShared.Status status)
        {
            mStatus = status;
        }

		public void on(App a)
		{
			// Quietly fail if anyone earlier couldn't do their job.
			if (string.IsNullOrEmpty(a.DownloadDestination))
			{
				return;
			}

            mStatus.WriteLine("Install " + a.Local);

			// Clear out the directory.  Have a minimum level of assurance
			// that we're not deleting something important.
			if (a.Local.Length < 12)
			{
				throw new DsException("WARNING:  Being asked to delete a folder path less then 12 chars long, that seems dangerous!");
			}

			Directory.CreateDirectory(a.Local);

            string tmpFolder = GetUniqueFolder();

			using (ZipFile zip = ZipFile.Read(a.DownloadDestination))
			{
				foreach (ZipEntry e in zip)
				{
                    e.Extract(tmpFolder);
				}
			}
            // The resulting folder should always have a extracted folders (should
            // just be one, but I'll plan for multiples) inside it,
            // so replace the destination with this.
            string[] src = Directory.GetDirectories(tmpFolder);
            if (src != null)
            {
                foreach (string srcDir in src)
                {
                    string s = Path.GetFileName(srcDir);
                    string d = Path.Combine(a.Local, s);
                    if (Directory.Exists(d))
                    {
                        Directory.Delete(d, true);
                    }
                    Directory.Move(srcDir, d);
                }
            }

            // Delete what we downloaded
			File.Delete(a.DownloadDestination);
            // Delete where we did the unzipping
            Directory.Delete(tmpFolder, true);
		}

        private static string GetUniqueFolder()
        {
            string ans;
            while (Directory.Exists(ans=GetRandomFolder())) ;
            Directory.CreateDirectory(ans);
            return ans;
        }

        private static string GetRandomFolder()
        {
            return Path.Combine(Path.GetTempPath(), "ds_" + Path.GetRandomFileName());
        }
    }
}
