﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Threading;

using DsInstaller.model;

using DsShared;

namespace DsInstaller.behaviour
{
	// Locate the remote file to download.  Responsible for
	// setting a valid downloadSource property on an app.
	class Locator
	{
		private NetworkCredential Credential = null;

		public Locator(NetworkCredential c)
		{
			Credential = c;
		}

		public void on(App a)
		{
			try
			{
				privateOn(a);
			}
            catch (System.Net.WebException ex)
            {
                Console.WriteLine("ERROR:  System.Net.WebException " + ex.Message + " response=" + ex.Response.ToString());
                Thread.Sleep(5000);
            }
			catch (System.Exception ex)
			{
				Console.WriteLine("ERROR:  Could not locate remote file (" + a.Remote + ") version (" + a.Version + ")");
                Console.WriteLine("exception=" + ex.Message + " " + ex.GetType());
                Thread.Sleep(5000);
            }
		}

		private void privateOn(App a)
		{
			DsShared.NetworkLocation loc = new DsShared.NetworkLocation(a.Remote);
			string remote = loc.ComposePath();

			string[] dir = DsShared.NetUtil.FtpGetRemoteDirectoryContents(Credential, remote);
			SortedList<DsShared.Version, string> all = DsShared.Version.getFromFiles(dir);
			string found = null;
			if (string.IsNullOrEmpty(a.Version))
			{
				if (all.Count > 0)
				{
					found = all.Values[all.Count - 1];
				}
			}
			else
			{
				all.TryGetValue(new DsShared.Version(a.Version), out found);
			}
			if (string.IsNullOrEmpty(found))
			{
				throw new DsException("No remote version " + a.Version);
			}
			a.DownloadSource = remote + NetUtil.BACK_SZ + found;
            a.DownloadVersion = DsShared.Version.newFromFile(found);
		}
	}
}
