﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Xml;

namespace DsInstaller.model
{
	class Package
	{
		public List<App>		mApp = new List<App>();

		// Answer a collection of packages
		static public Package[] ReadXmlAllFrom(string[] filesOrFolders)
		{
            // Flatten the input into a list of files.
            List<string>    files = new List<string>();
            foreach (string f in filesOrFolders)
            {
                if (Directory.Exists(f))
                {
                    string[] list = Directory.GetFiles(f, "*.xml");
                    if (list != null)
                    {
                        foreach (string s in list)
                        {
                            files.Add(s);
                        }
                    }
                }
                else if (File.Exists(f))
                {
                    string ext = Path.GetExtension(f);
                    if (ext == ".xml" || ext == ".XML")
                    {
                        files.Add(f);
                    }
                }
            }
            return ReadXmlAllFromFiles(files.ToArray());
		}

        // Answer a collection of packages from a list of files.
        static public Package[] ReadXmlAllFromFiles(string[] filePaths)
        {
            if (filePaths == null || filePaths.Length < 1)
            {
                return null;
            }
            Package[] ans = new Package[filePaths.Length];
            for (int k = 0; k < filePaths.Length; ++k)
            {
                ans[k] = new Package();
                ans[k].ReadXmlFrom(filePaths[k]);
                Console.WriteLine("input " + filePaths[k]);
            }
            return ans;
        }


		public void ReadXmlFrom(string fn)
		{
			XmlTextReader reader = new XmlTextReader(fn);

			// Just skip xmls that are the wrong type
			if (!reader.Read() || reader.Name != "package")
			{
				return;
			}

			while (reader.Read())
			{
				switch (reader.NodeType)
				{
					case XmlNodeType.Element: // The node is an element.
						if (reader.Name == "app")
						{
							App app = new App();
							if (app.ReadXmlFrom(reader))
							{
								mApp.Add(app);
							}
						}
						break;
					case XmlNodeType.Text:
						break;
					case XmlNodeType.EndElement:
						break;
				}
			}
		}
	}
}
