﻿using System;
using System.Collections.Generic;

namespace DsShared
{
	// A component representation of our standardized version string,
	// consisting of the form M.m.b (Major.minor.bug, i.e. 1.0.1)
	class Version:IComparable<Version>
	{
		public int Major { get; set; }
		public int Minor { get; set; }
		public int Bug { get; set; }

		// Given a list of version strings, answer a sorted list of
		// assocations from Version objects to their original strings.
		// The members of string are filenames.
		public static SortedList<Version, string> getFromFiles(string[] all)
		{
			SortedList<Version, string> ans = new SortedList<Version, string>();
			if (all != null)
			{
				foreach (string a in all)
				{
					ans.Add(Version.newFromFile(a), a);
				}
			}
			return ans;
		}

		public static Version newFromFile(string filename)
		{
			int pos = filename.IndexOf('.');
			if (pos != -1)
			{
				filename = filename.Substring(0, pos);
			}
			filename = filename.Replace('_', '.');
			return new Version(filename);
		}

		public Version()
		{
			Clear();
		}

		public Version(string v)
		{
			Clear();
			string[] all = v.Split('.');
			if (all != null && all.Length > 0 && !string.IsNullOrEmpty(all[0])) Major = Convert.ToInt32(all[0]);
			if (all != null && all.Length > 1 && !string.IsNullOrEmpty(all[1])) Minor = Convert.ToInt32(all[1]);
			if (all != null && all.Length > 2 && !string.IsNullOrEmpty(all[2])) Bug = Convert.ToInt32(all[2]);
		}

		public void Clear()
		{
			Major = 0;
			Minor = 0;
			Bug = 0;
		}

		public int CompareTo(Version other)
		{
			if (Major < other.Major) return -1;
			else if (Major > other.Major) return 1;

			if (Minor < other.Minor) return -1;
			else if (Minor > other.Minor) return 1;

			if (Bug < other.Bug) return -1;
			else if (Bug > other.Bug) return 1;

			return 0;
		}

        public override string ToString()
        {
            string ans = "";
            ans = ans + Convert.ToString(Major) + "." + Convert.ToString(Minor) + "." + Convert.ToString(Bug);
            return ans;
        }
	}
}
